program test_superlu
  use laplacian_mod
  use superlu_mod
  implicit none

  type(CSR) :: A
  real(kind=8), pointer, dimension(:) :: b, x
  INTEGER :: nx, ny
  REAL(kind=8) :: dx, dy

  !!!!
  !! init data
  !!!!
  nx = 501
  ny = 501
  dx = 1.d0/(nx+1)
  dy = 1.d0/(ny+1)

  allocate(b(nx*ny), x(nx*ny))
  b = 1.d0
  x = 1.d0

  !!!!
  !! put Dirichlet condition in the second member
  !!!!
  call setDirichlet(b, nx, ny, dx, dy)

  !!!!
  !! Laplacian matrix construction 
  !! without Dirichlet points
  !!!!
  call LaplacianMatrix(A, nx, ny, dx, dy)

  !!!!
  !! solve Ax = b with SuperLU library
  !!!!
  call superlu_solver(A, b, x)  

  !!!!
  !! save x in VTK format
  !!!!
  call save_vtk(nx, ny, dx, dy, x,"test_superlu.vts")

  !!!!
  !! Destroy A, b and x
  !!!!
  call destroyCSR(A)
  deallocate(b, x)

end program test_superlu
